# DOC
#
#  server.tcl  - very simple server application in Tcl,
#                to see how this might work
#
#  Copyright (C) 2001 Arjen Markus
#
#  Arjen Markus
#
#
#  General information:
#  This script sets up a server that listens to a socket.
#  It prints whatever it receives and does no more than that.
#
#  It contains the following procedures:
#  - SetUp:          Create the server
#  - Accept:         Accept the service request
#  - PrintInput:     Print the received input
#  - (main code):    Start up of everything
#
# ENDDOC
#
#  $Author$
#  $Date$
#  $Source$
#  $Log$
#

# --------------------------------------------------------------------
#   Procedure: SetUp
#   Author:    Arjen Markus
#   Purpose:   Create the server
#   Context:   Used in main code
#   Summary:
#              Set up the server socket and enter the event loop
#   Arguments:
#              -- none --
#   Return value:
#              -- none --
# --------------------------------------------------------------------
#
proc SetUp  { } {
   set port    8085
   set timeout 60000

   socket -server [list Accept $timeout] $port
   UpdateScreen
}

proc UpdateScreen { } {
   after 100 { update idletasks ; UpdateScreen }
}

# --------------------------------------------------------------------
#   Procedure: Accept
#   Author:    Arjen Markus
#   Purpose:   Accept the service request
#   Context:   Used by server script
#   Summary:
#              Set up the call-back script
#   Arguments:
#   timelimit  Maximum time to wait
#   socket     Name of the socket we listen to
#   ip         IP address of client
#   args       (Possibly) all other arguments
#
#   Return value:
#              -- none --
# --------------------------------------------------------------------
#
proc Accept { timelimit socket ip args } {
   upvar #0 $socket request

   fconfigure $socket -block false
   fileevent  $socket readable [list DrawInput $socket]
}

# --------------------------------------------------------------------
#   Procedure: DrawInput
#   Author:    Arjen Markus
#   Purpose:   Print the received input
#   Context:   Used by server script
#   Summary:
#              Simply print the input to the terminal window
#   Arguments:
#   socket     Name of the socket we listen to
#
#   Return value:
#              -- none --
# --------------------------------------------------------------------
#
proc DrawInput { socket } {
   upvar #0 $socket request
   global xprev yprev

   if { ! [eof $socket] } {
      gets $socket line
      puts $line
      set x [lindex $line 0]
      set y [lindex $line 1]

      if { $x != "" && $y != "" } {
         set xc [expr 100+80*$x]
         set yc [expr 100+80*$y]
         .c create line $xprev $yprev $xc $yc -fill black
         set xprev $xc
         set yprev $yc
      }
   }
}

# --------------------------------------------------------------------
#   Procedure: main code
#   Author:    Arjen Markus
#   Purpose:   Start the server
#   Context:   --
#   Summary:
#              Simply call the server SetUp script
# --------------------------------------------------------------------
#

global xprev yprev

set xprev 0.0
set yprev 0.0

canvas .c -background white
pack .c

SetUp

